using Nemerle;
using Nemerle.Compiler;
using Nemerle.Compiler.Parsetree;
using Nemerle.Utility;

namespace Oyster.Units.Macros
{
	/// Base class for literal macro-operators
	public class UnitLiteralMacro : IMacro
	{
		_opName : string;
		_unitName : string;

		/// Creates new macro-operator instance
		public this(opName : string, unitName : string)
		{
			_opName = opName;
			_unitName = unitName
		}

		/// Main run method
		public Run(_ : Typer, l : list[SyntaxElement]) : PExpr
		{
			match (l) {
				| [ SyntaxElement.Expression(expr) ] =>
					// Ctor call here
					PExpr.Call(Util.UseSiteExpr(_unitName), [ expr ])
				| _ =>
					Message.FatalError(
						"macro `$(GetName())' expects following list of arguments: (PExpr) "
						"got some $(l.Length) parameters [$(l.ToString(\", \"))]")
			}
		}

		// Name and namespace
		public GetName() : string
		{
			_opName
		}

		public GetNamespace() : string
		{
			Util.MacrosNS
		}

		// This below is only to satisfy the interface requirements.
		public CallTransform(l : list[PExpr]) : list[SyntaxElement]
		{
			l.Map(SyntaxElement.Expression(_))
		}

		public SyntaxExtension() : GrammarElement * (list[SyntaxElement] -> list[SyntaxElement])
		{
			(null, fun(x) { x })
		}

		public IsInherited : bool
		{
			get { false }
		}

		public IsTailRecursionTransparent : bool
		{
			get { false }
		}

		public Keywords : list[string]
		{
			get { [] }
		}

		public Usage : MacroUsageAttribute
		{
			get { null }
		}
	}
}
